#include <iostream>
#include <iomanip>
using namespace std;

// This is a class to store matrices.  Note that this class does not contain
// operators to do arithmetic, but a useful Matrix class should.  It's
// not part of this assignment, however.  It is called "operator overloading"
// and is something you can learn to do in an object-oriented programming course
class Matrix {
public:
    Matrix();                // default constructor
    Matrix(int nr, int nc);  // constructor - use if you know initial size
    // Insert here the prototypes of ANY additional functions that are
    // required, given that this class dynamically allocates memory
    bool set(int r, int c, float v);  // set the value of a single element
    void resize(int r, int c);        // change the size of the matrix
    void display(ostream &o) const;   // output the matrix in rows and columns
private:
    int nrows;   // number of rows in the matrix
    int ncols;   // number of columns in the matrix
    float *data; // data should point to an array of floats.  This array
                 // stores all the elements of the array.  Matrix objects
                 // own this memory
};

// Insert here the implementation of all member functions in the Matrix class

// Insert here an overloaded stream insertion operator, so that if m1 and
// m2 are Matrix objects, then we can display them with the line
// cout << m1 << m2;

// Here is a little test program for the Matrix class.
// You don't need to alter this code, but should completely understand
// every line.  Make sure in particular that you know exactly when
// every destructor, assignment operator and constructor are called (and
// which constructor is called.

void foo(Matrix m);

int main()
{
    int i, j;
    Matrix m1;         // statically allocate a Matrix object. Starts out 0x0 
    Matrix m2(3, 4);   // statically allocate a 3x4 matrix object
    for (i=0; i < 3; i++)
        for (j=0; j < 4; j++)
            m2.set(i, j, i*4+j);              // set all values in m2
    Matrix m3(m2);                            // create m3 to be a copy of m2
    m1 = m2 = m2;                             // store m2 in m1
    cout << "Matrix m1, Matrix m2, and Matrix m3: " << endl;
    cout << m1 << endl << m2 << endl << m3 << endl;   // output all 3 matrices

    float v;
    int nr, nc;
    Matrix *mp;          // create a pointer to a matrix
    mp = new Matrix;     // dynamically allocate a matrix
    cout << "Enter the number of rows and columns in your matrix: ";
    cin >> nr >> nc;
    mp->resize(nr, nc);  // set a new size for matrix
    cout << "Enter " << nr*nc << " values for the contents of the array: ";
    for (i=0; i < nr; i++) {
        for (j=0; j < nc; j++) {
            cin >> v;            // input values from the user
            mp->set(i, j, v);    // and store in the matrix
        }
    }
    // output the entire matrix
    cout << "This is your matrix: " << endl;
    cout << *mp << endl;
    foo(*mp);

    delete mp;      // release memory (note: destructor is called automatically)

    return 0;
}

void foo(Matrix m)
{
    cout << "Matrix object passed to a function: " << endl;
    cout << m << endl;
}
